'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.list = list;
exports.create = create;
exports.get = get;
exports.update = update;
exports.destroy = destroy;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _queryBuilder = require('./query-builder');

var _queryBuilder2 = _interopRequireDefault(_queryBuilder);

var _objectPath = require('object-path');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

/*** ---CRUD Methods--- ***/

/** Query resource list **/

function list(model, req, cb) {
  return _bluebird2['default'].resolve(req.query.filter).then(function (body) {
    return model.lifecycle.beforeAccess(body, req);
  }).then(function (body) {
    return (0, _queryBuilder2['default'])(model, body);
  }).then(function (body) {
    return model.lifecycle.afterAccess(body, req);
  }).nodeify(cb);
}

/** Create new resource **/

function create(model, req, cb) {
  var body = req.body;

  return _bluebird2['default'].resolve(body).then(function (body) {
    return model.lifecycle.beforeSave(body, req);
  }).then(function (body) {
    return model.lifecycle.beforeCreate(body, req);
  }).then(function (body) {
    return model.create(body);
  }).then(function (body) {
    return model.lifecycle.afterSave(body, req);
  }).then(function (body) {
    return model.lifecycle.afterCreate(body, req);
  }).nodeify(cb);
}

/** Return a single resource by ID **/

function get(model, req, cb) {
  var query = req.query;
  var params = req.params;

  (0, _objectPath.set)(query, 'filter.where.id', params.id);
  return list(model, req, cb);
}

/** Update a single resource by ID **/

function update(model, req, cb) {
  var body = req.body;
  var params = req.params;

  return _bluebird2['default'].resolve(body).then(function (body) {
    return model.lifecycle.beforeSave(body, req);
  }).then(function (body) {
    return model.lifecycle.beforeUpdate(body, req);
  }).then(function (body) {
    return model.update({ id: params.id }, body);
  }).then(isSingle).then(function (body) {
    return model.lifecycle.afterSave(body, req);
  }).then(function (body) {
    return model.lifecycle.afterUpdate(body, req);
  }).nodeify(cb);
}

/** Destroy a single resource by ID **/

function destroy(model, req, cb) {
  var query = req.query;
  var params = req.params;

  return _bluebird2['default'].resolve(query.filter).then(function (body) {
    return model.lifecycle.beforeDelete(body, req);
  }).then(function () {
    return model.destroy({ id: params.id });
  }).then(function (body) {
    return model.lifecycle.afterDelete(body, req);
  }).nodeify(cb);
}

function isSingle(results) {
  if (Array.isArray(results) && results.length === 1) return results[0];
  return results;
}