'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _waterline = require('waterline');

var _waterline2 = _interopRequireDefault(_waterline);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _async = require('async');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash');

var _objectPath = require('object-path');

var _changeCase = require('change-case');

var METHODS = [
// GET
'beforeAccess', 'afterAccess',

// PUT, POST
'beforeSave', 'afterSave',

// PUT
'beforeUpdate', 'afterUpdate',

// POST
'beforeCreate', 'afterCreate',

// DELETE
'beforeDelete', 'afterDelete'];

var passThrough = function passThrough(data) {
  return _bluebird2['default'].resolve(data);
};

exports['default'] = function (config) {
  // Instantiate a new instance of the ORM
  var orm = new _waterline2['default']();

  var schema = _glob2['default'].sync(_path2['default'].join(config.dir, '**/*.js')).map(require).reduce(function (schema, schemata) {
    schema[schemata.identity] = schemata;
    return schema;
  }, {});

  Object.keys(schema).map(getModel).map(function (model) {
    model.lifecycle = getLifecycle(model);

    model = (0, _lodash.merge)({}, config.defaults, model);

    if (!model.tableName) model.tableName = (0, _changeCase.pascalCase)(model.identity);
    return model;
  }).filter(function (model) {
    return model['public'] !== false && !!model.connection;
  }).map(function (model) {
    return _waterline2['default'].Collection.extend(model);
  }).map(orm.loadCollection.bind(orm));

  return _bluebird2['default'].promisify(orm.initialize.bind(orm))(config).then(function (models) {
    return models.collections;
  });

  function getModel(name) {
    var obj = schema[name];
    if (!obj) throw new Error('Schmemata for model ' + name + ' not found');

    if (obj.base) obj = (0, _lodash.merge)({}, getModel(obj.base), obj, function (a, b, key) {
      if (~METHODS.indexOf(key) && a && b) {
        if (!Array.isArray(a)) a = [a];
        if (!Array.isArray(b)) b = [b];
        return a.concat(b);
      }
    });

    return obj;
  }

  function getLifecycle(model) {
    return METHODS.reduce(function (prev, method) {
      var fn = (0, _objectPath.get)(model, ['lifecycle', method], passThrough);
      (0, _objectPath.set)(prev, method, execLifecycle.bind(model, fn));
      return prev;
    }, {});
  }

  function execLifecycle(lifecycle) {
    var _this = this;

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var instance = args[0];

    if (!instance) instance = args[0] = {};
    if (!lifecycle) lifecycle = function () {
      return _bluebird2['default'].resolve(instance);
    };
    if (!Array.isArray(lifecycle)) lifecycle = [lifecycle];

    return _bluebird2['default'].fromNode(function (cb) {
      (0, _async.reduce)(lifecycle, instance, function (i, mw, cb) {
        _bluebird2['default'].method(mw).apply(_this, args).then(function (ins) {
          if (typeof ins !== 'undefined') return ins;
          return instance;
        }).nodeify(cb);
      }, cb);
    });
  }
};

module.exports = exports['default'];